<style type="text/css">
* {
	font-family: "Helvetica Neue", Arial, Helvetica, Geneva, sans-serif;
}

h1 {
	border-bottom: 1px solid #003399;
	font-size: 36px;
	font-variant: normal;
	text-decoration: none;
	text-transform: uppercase;
}

h2 {
	font-size: 24px;
	color: #5e5e5e;
}

h3 {
	font-size: 18px;
	text-decoration: underline;
}

h4 {
	font-size:12px;
	font-weight:bold;
}

div.descriptor {
	background-color: #eaeaea;
	padding: 10px;
	margin: 10px;
}

table.params {
	width: 100%;
}

table.params th {
	text-align: left;
}

table.params td {
	background-color: #fffeff;
	font-size: 13px;
	font-style: italic;
	padding: 5px;
}

div.descriptor h1 {
	font-size: 24px;
}

div.descriptor h2 {
	font-size: 18px;
}

div.docs {
	background-color: #fffeff;
	padding: 10px;
	margin-top: 10px;
}

div.resource {
	padding: 10px;
	margin: 10px;
	border: 1px solid #797979;
	background-color: #efefef;
}

div.resource h2 {
	color: #212121;
}

div.resource h3 {
	text-decoration: none;
	background-color: #f6f6f6
	;
}

div.toc {
	font-size:12px;
	background-color: #f9f9f9;
	border: 1px solid #797979;
	padding: 10px;
}

.tiny {
	font-size:10px;
}
</style>